/*Pathrider Instrumentaton
  ------------------------  
  Initially a merging of DS3231_only + CELSIUS_multiple

  Transmits temperature values from 5 dallas sensors as text over BT serial to the mobile App (PathRider1.aia). 
  The string is delimited with a hash (#) at start, then commas between each temperature reading and a line 
  feed terminator, as follows: 
                #  DT0   DT1   DT2   DT3   DT4 \n
  For example: "#,00.00,11.11,22.22,33.33,44.44\n"

  The string length should always be 1 + (5 x 6) + 1 (linefeed) = 32

  If a thermometer is detected but hasn't been allocated a DTx, or a DTx allocated device has not 
  been connected, a negative value ( > -10 ) will be transmitted: 
  For example: "#,11.11,22.22,33.33,-8.00,-9.00\n" 


-------------------------------------------------------------------------------------------------------------
22Nov[k] changed defaults for DTx_degrees from -99 to -9, -8, -7, -6, -5 to distinguish each.
22Nov[j] added comma after "#" in serial transmission 
19Nov[i] interval reduces to 2 sec 
19Nov[h] loop() now @ 5 sec interval
12Nov[g] Updated introduction.
09Nov[f] added transmission of multiple temperatures as a CSV string. Not tested yet.
07Nov[e] 12.34°C testing, added steadily increasing DT0 value in gatherCelsiusD0(){}
06Nov[d] reinstated loop() replacing "dual echo loop" Added BT sending float 
01Nov[c] removed timers/TMR tabs added BT but not working yet
31Oct[b] added showCelsius()
31Oct[a] 2024. Created by merging DS3231_only.m + CELSIUS_multiple.d
============================================================================================================= */

#include "ZZ.h"                       // global #defines
#include "Cels.h"                     // temperatures
#include "OTXT.h"                     // OLED 
#include "PTC.h"                      // DS3231 only 
//nclude "TMR.h"                      // Timer using TeensyTimerTool 
#include "xMSC.h"                     // Menu etc

#define bt Serial7                    // Bluetooth on TX7/RX7 
#define bt_baud 38400                 // CJ bt modules all set to 38400 (also 38400 is mandatory in AT/config. mode)

uint32_t sec_0, sec_1;                // clock seconds  

// fwd decl.
void every2secs();

char *sketch = truncateString(FILENAME,4);

// ====================================================================================================
void setup(){
  pinMode(30,OUTPUT); // dim yellow BT LED on T4.1 expansion board  
  pinMode(LEDpin , OUTPUT); 
  pinMode(LEDpin2, OUTPUT); flashLED(2);
  pinMode(TONEpin, OUTPUT); beep2();
  Serial.begin(115200);                   // not required for teensy's
  while (!Serial && millis() < 1000) ;    // wait for serial, up to 1 sec
  Serial << '\n' << '\n' << CF(dashes) << sketch << CJ_V << CF(dashes) << '\n';
  initOTXT(0);                            // 0 = scrolling off, 1 = scrolling on
  initRTC();
  Serial << F(" * Initialise Bluetooth ......... : ");            // tell receiver we're awake
  bt.begin(bt_baud);
  if (!bt) {Serial << "** FAIL ** ... program HALTED.\n"; while(1);} 
  Serial << "OK\n";
  delay(10);
  bt.flush();                             // clear any BT backlog in serial buffer 
  initCelsius();                          // Dallas temperature sensors
  Serial << "----------- 0 -----------\n";
  reportCELSIUS();
  printMenu();
  otxt.clear();                           // reset OLED cursor to 0,0
  show_RTC_ID(0,0,1);                     // display RTC_ID on OLED
  Serial << '\n' << CF(dashes) << F("Setup done") << CF(dashes) << '\n'<< '\n'; 
  sec_0 = now();                          // start timestamp
  }

unsigned long loopCount = 0, count1s = 0, count2s = 0;
time_t timeB;
bool flag = false;

void loop(){
  //if (Serial.available()) { char c = Serial.read(); bt.write(c); }
  //if (bt.available())     { char b = bt.read(); Serial.write(b); }  
  if (loopCount % 100 == 0) processSerialCommands();
  sec_1 = now();
  if (sec_1 - sec_0 >= 2){
    count1s++;
    flashLED(1);
    processTime();                                                    // calculates timeX and timeC
    showDate(timeC,4*6,0,1);                                          // show date on OLED
    showTime(timeC,15,2,2);                                           // add time 
    otxt.set1X(); 
    timeB = timeC - start_time;                                       // time since boot up
    //otxt.setCursor(0,5); otxt << "elapsed:"; showTime(timeB,13*6,5,1);// add duration since boot up
    timeSinceSet = now() - timeWhenSet;
    //otxt.setCursor(0,7); otxt << "since:"; showTimeSinceSet(timeSinceSet,60,7,1);
    showCelsius(DT0_degrees,  0,6,1);                                  // display DT0 value in OLED
    showCelsius(DT1_degrees, 26,7,1);                                  // and DT1 
    showCelsius(DT2_degrees, 52,6,1);                                  // and DT2 
    showCelsius(DT3_degrees, 78,7,1);                                  // and DT3 
    showCelsius(DT4_degrees,104,6,1);                                  // and DT3 
    bt  << "#" << ","
        << _FLOAT(DT0_degrees,2) << ","
        << _FLOAT(DT1_degrees,2) << ","    
        << _FLOAT(DT2_degrees,2) << ","    
        << _FLOAT(DT3_degrees,2) << ","    
        << _FLOAT(DT4_degrees,2) << '\n';                              // send to mobile over bluettoth  
    if (!VERBOSE) {
      reportDateTime(timeC);
      Serial << '\t' << DT0_degrees 
             << '\t' << DT1_degrees 
             << '\t' << DT2_degrees 
             << '\t' << DT3_degrees 
             << '\t' << DT4_degrees << ' ' << DEGREES_C << '\n';} 
    if (flag) every2secs();                                           // do extra stuff every 2 secs
    sec_0 = sec_1;
    flag = !flag;
    }
  delay(1);
  loopCount++;
  }

void every2secs(){
  count2s++;
  int count2sMod4 = count2s % 4;
  timeSinceSet = now() - timeWhenSet;                                 // time since RTC set
  primeCELSIUS();  
  gatherCELSIUS();
  if (VERBOSE) {
    Serial << F("\n==== T2000 [") << count1s << '/' << count2s << '/' << count2sMod4 << ']' << F(" === loops: ") << loopCount << F(" ====\n");
    Serial << F("timeB\t"); report_HHMMSS(timeB); Serial << '\n';
    Serial << F("set  \t"); reportDateTime(timeWhenSet);
    Serial << F("since\t"); reportTimeSinceSet(timeSinceSet); Serial << '\n';    
    Serial << F("timeX\t"); reportDateTime(timeX);
    reportCELSIUS();
    }
  }


/* dual echo loop
void loop(){
  if (Serial.available()) { char c = Serial.read(); bt.write(c); }
  if (bt.available())     { char b = bt.read(); Serial.write(b); }  
  }
*/

// ---- TImers --------------
  /*
  // in setup()
  #ifdef TIMERS_ACTIVE
    Serial << " * TIMERS_ACTIVE\n";
    startPeriodTimer(500'000);            // µs = 1Hz
    startPeriodTimer2(1'000'000);         // µs = 2Hz
  #endif
  */

  /*
  // in loop()
  #ifdef TIMERS_ACTIVE
    if (count1s ==  5) {Serial << '\n' << count1s << ": "; adjustPeriodTimer(  100'000);}  // (µs) 5Hz
    if (count1s ==  7) {Serial << '\n' << count1s << ": "; adjustPeriodTimer(   50'000);}  // (µs) 10Hz
    if (count1s ==  9) {Serial << '\n' << count1s << ": "; adjustPeriodTimer(   20'000);}  // (µs) 25Hz
    if (count1s == 11) {Serial << '\n' << count1s << ": "; adjustPeriodTimer(    5'000);}  // (µs) 100Hz
    if (count1s == 13) {Serial << '\n' << count1s << ": "; adjustPeriodTimer(1'000'000);}  // (µs) 0.5Hz (2s)
    if (count1s == 15) {Serial << '\n' << count1s << ": "; adjustPeriodTimer(  500'000);}  // (µs)  1Hz
    // demo OneShotTimer
    if (count1s == 17) {Serial << '\n' << count1s << ":\tStop period timer & turn LED off\n\n"; 
                        stopPeriodTimer(); digitalWriteFast(LEDpin, LOW);}
    if (count1s == 20) {Serial << '\n' << count1s << ": "; startOneShotTimer(200'000);}   // (µs)
    if (count1s == 25) {Serial << '\n' << count1s << ": "; startPeriodTimer(500'000); }   // (µs)
  #endif
  #ifndef TIMERS_ACTIVE
    flashLED(1);
  #endif
  */
